// Hat.cpp: implementation of the CHat class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Std.h"
#include "Hat.h"
#include "math.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif


// Bezier resolution when editing
// # pnts pr curvesegment
#define BEZ_EDIT 50 // must be even

// steps in rotation
#define ROT_STEP 22

// the logical coordinaterange of an objectdescription
#define BASIC_SIZE 1000

// since OGL and WAPI want angles in diff format:
#define GtR	0.01745277777778f
#define PI  3.14159265359f

// to identify single controlpoints
#define HITSQUARE max(BASIC_SIZE/100,2)


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CHat::CHat()
{
	z = 0.0f;
	vx = 40.0f;

	Con.RemoveAll(); // to make sure
	Bez.RemoveAll(); // to make sure

	P3Dn p;
	p.x=-195; p.y=0; p.z=-180; Con.Add(p);
	p.x=-80; p.y=0; p.z=-260; Con.Add(p);
	p.x=-195; p.y=0; p.z=0; Con.Add(p);
	p.x=-100; p.y=0; p.z=-50; Con.Add(p);

	//p.x=-195; p.y=0; p.z=0; Con.Add(p);
	//p.x=-100; p.y=0; p.z=-50; Con.Add(p);

	p.x=-130; p.y = 0; p.z = -50; Con.Add(p);
	p.x=-80; p.y = 0; p.z = -50; Con.Add(p);
	p.x=0; p.y=0; p.z = -50; Con.Add(p);
	p.x=0; p.y=0; p.z = -50; Con.Add(p);


	m_IsMoving=FALSE;
	m_MovePix=-1;

	//hilited=-1;
}

CHat::~CHat()
{

}

void CHat::makeCompleteB()
{
	// produce complete Beziercontour Bez from 
	// controlpnts Con
	// controlpoints are long within BASIC_SIZE
	// Bezier will be float within 1.0f
	
	// NB!!! watch the yz-conversion below

	// prepare floatdescriptions
	Bez.SetSize(BEZ_EDIT,BEZ_EDIT);
	
	// and build from scratch
	int remaining=Con.GetSize();
	int pix=0;
	P3Dn p0,p1,p2,p3;
	P3Df pf;
	while(remaining>=4)
	{
		p0=Con[pix];
		p1=Con[pix+1];
		p2=Con[pix+2];
		p3=Con[pix+3];
		remaining-=4;
		pix+=4;
		// ok, the controlpoints are p0,p1,p2,p3

		// and the bezier curve
		float xf,yf,zf;
		float t1=0.0f;
		float t2=0.0f;
		float t3=0.0f;
		for(int bix=0;bix<BEZ_EDIT;bix++)
		{
			// can be speeded up with B-tables
			xf=	(1.0f-3*t1+3*t2-t3)*p0.x+
				(3*t1-6*t2+3*t3)*p1.x+
				(3*t2-3*t3)*p2.x+
				t3*p3.x;
			yf=	(1.0f-3*t1+3*t2-t3)*p0.y+
				(3*t1-6*t2+3*t3)*p1.y+
				(3*t2-3*t3)*p2.y+
				t3*p3.y;
			zf=	(1.0f-3*t1+3*t2-t3)*p0.z+
				(3*t1-6*t2+3*t3)*p1.z+
				(3*t2-3*t3)*p2.z+
				t3*p3.z;

			pf.x=xf/(1.0f*BASIC_SIZE);
			pf.y=yf/(1.0f*BASIC_SIZE);
			pf.z=zf/(1.0f*BASIC_SIZE);
			Bez.Add(pf);

			t1+=1.0f/(1.0f*BEZ_EDIT);
			t2=t1*t1;
			t3=t2*t1;
		}
	}
}

void CHat::draw()
{
	// draws a rotationobject based on
	// contour Cont, in xz-plane rotate around y-axes
	
	// Produce complete Bezierpoints
	//glTranslatef(0.0f, 0.0f, z);
	makeCompleteB();

	float x1b,y1b,z1b,x2b,y2b,z2b;	// basic strip,
	float x1,y1,z1,x2,y2,z2;		// running points
	float x1l,y1l,z1l,x2l,y2l,z2l;	// last strip

	glDisable(GL_CULL_FACE);
	glPushMatrix();
 	glEnable(GL_NORMALIZE);
	int count=Bez.GetSize();
	int pix=0;
	while(pix < count-1)
	{

		x1b=Bez[pix].x;		y1b=Bez[pix].y;		z1b=Bez[pix].z;
		x2b=Bez[pix+1].x;	y2b=Bez[pix+1].y;	z2b=Bez[pix+1].z;
		
		// the basc points as described in contour window
		// (x1b,y1b,z1b),(x2b,y2b,z2b)

		// run round with this strip
		float v=0.0f;
		float dv=2.0f*PI/(1.0f*ROT_STEP);
		// need last pnt to calculate normals
		// initialized as prev to first, same as last
		x1l=x1b*(float)cos(2.0f*PI-dv)-y1b*(float)sin(2.0f*PI-dv);
		y1l=x1b*(float)sin(2.0f*PI-dv)+y1b*(float)cos(2.0f*PI-dv);
		z1l=z1b;
		x2l=x2b*(float)cos(2.0f*PI-dv)-y2b*(float)sin(2.0f*PI-dv);
		y2l=x2b*(float)sin(2.0f*PI-dv)+y2b*(float)cos(2.0f*PI-dv);
		z2l=z2b;

		int lix=0;
		glBegin(GL_QUAD_STRIP);
		while(lix<=ROT_STEP)
		{
			x1=x1b*(float)cos(v)-y1b*(float)sin(v);
			y1=x1b*(float)sin(v)+y1b*(float)cos(v);
			z1=z1b;

			x2=x2b*(float)cos(v)-y2b*(float)sin(v);
			y2=x2b*(float)sin(v)+y2b*(float)cos(v);
			z2=z2b;

			// cross product (a2b3-a3b2)(a3b1-a1b3)(a1b2-a2b1)
			// normal based on (p1->p1l) X (p1->p2)
			glNormal3f(	-((y1l-y1)*(z2-z1)-(z1l-z1)*(y2-y1)),
						-((z1l-z1)*(x2-x1)-(x1l-x1)*(z2-z1)),
						-((x1l-x1)*(y2-y1)-(y1l-y1)*(x2-x1))	);
			//glNormal3f(x1,y1,0.0f);
			glVertex3f(x1,y1,z1);

			// normal based on (p2->p1) X (p2->p2l)
			glNormal3f(	((y2-y1)*(z2l-z2)-(z2-z1)*(y2l-y2)),
						((z2-z1)*(x2l-x2)-(x2-x1)*(z2l-z2)),
						((x2-x1)*(y2l-y2)-(y2-y1)*(x2l-x2))	);
			//glNormal3f(x2,y2,0.0f);
			glVertex3f(x2,y2,z2);

			// remember last
			x1l=x1; y1l=y1; z1l=z1;
			x2l=x2; y2l=y2; z2l=z2;

			lix++;
			v+=dv;
		}
		glEnd();
		
		if((pix+2)%BEZ_EDIT==0)
			pix++;
		pix++;
	}
	glPopMatrix();
	glEnable(GL_CULL_FACE);
}
